@@ -1,5 +1,3 @@
-# -*-perl-*-
-
 package Astro::FITS::HdrTrans::JCMT_GSD_DB;
 
 =head1 NAME
@@ -26,16 +24,16 @@ use base qw/ Astro::FITS::HdrTrans::Base /;
 
 use vars qw/ $VERSION /;
 
-$VERSION = sprintf("%d.%03d", q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "1.02";
 
 # for a constant mapping, there is no FITS header, just a generic
 # header that is constant
 my %CONST_MAP = (
-		 INST_DHS         => 'HET_GSD',
-		 COORDINATE_UNITS => 'decimal',
-		 EQUINOX          => 'current',
-		 TELESCOPE        => 'JCMT',
-		);
+                 INST_DHS         => 'HET_GSD',
+                 COORDINATE_UNITS => 'decimal',
+                 EQUINOX          => 'current',
+                 TELESCOPE        => 'JCMT',
+                );
 
 # NULL mappings used to override base class implementations
 my @NULL_MAP = ();
@@ -44,51 +42,51 @@ my @NULL_MAP = ();
 # to the output with only a keyword name change
 
 my %UNIT_MAP = (
-		AMBIENT_TEMPERATURE => "TAMB",
-		APERTURE => "APERTURE",
-		AZIMUTH_START => "AZ",
-		BACKEND => "BACKEND",
-		BACKEND_SECTIONS => "NORSECT",
-		CHOP_FREQUENCY => "CHOPFREQ",
-		CHOP_THROW => "CHOPTHRW",
-		COORDINATE_SYSTEM => "COORDCD",
-		#        COORDINATE_TYPE => "C4LSC",
-		CYCLE_LENGTH => "CYCLLEN",
-		#        DEC_BASE => "",
-		ELEVATION_START => "EL",
-		FILENAME => "GSDFILE",
-		FILTER => "FILTER",
-		FREQUENCY_RESOLUTION => "FREQRES",
-		FRONTEND => "FRONTEND",
-		HUMIDITY => "HUMIDITY",
-		NUMBER_OF_CYCLES => "NOCYCLES",
-		NUMBER_OF_SUBSCANS => "NOSCANS",
-		OBJECT => "OBJECT",
-		OBSERVATION_MODE => "OBSMODE",
-		OBSERVATION_NUMBER => "SCAN",
-		PROJECT => "PROJID",
-		#        RA_BASE => "C4RADATE",
-		RECEIVER_TEMPERATURE => "TRX",
-		ROTATION => "YPOSANG",
-		REST_FREQUENCY => "RESTFRQ1",
-		SEEING => "PHA",
-		SWITCH_MODE => "SWMODE",
-		SYSTEM_TEMPERATURE => "STSYS",
-		TAU => "TAU",
-		USER_AZ_CORRECTION => "UXPNT",
-		USER_EL_CORRECTION => "UYPNT",
-		VELOCITY => "VELOCITY",
-		VELOCITY_REFERENCE_FRAME => "VREF",
-		VELOCITY_TYPE => "VDEF",
-		X_BASE => "XREF",
-		Y_BASE => "YREF",
-		X_DIM => "NOXPTS",
-		Y_DIM => "NOYPTS",
-		X_REQUESTED => "XSOURCE",
-		Y_REQUESTED => "YSOURCE",
-		X_SCALE => "DELTAX",
-		Y_SCALE => "DELTAY",
-	       );
+                AMBIENT_TEMPERATURE => "TAMB",
+                APERTURE => "APERTURE",
+                AZIMUTH_START => "AZ",
+                BACKEND => "BACKEND",
+                BACKEND_SECTIONS => "NORSECT",
+                CHOP_FREQUENCY => "CHOPFREQ",
+                CHOP_THROW => "CHOPTHRW",
+                COORDINATE_SYSTEM => "COORDCD",
+                #        COORDINATE_TYPE => "C4LSC",
+                CYCLE_LENGTH => "CYCLLEN",
+                #        DEC_BASE => "",
+                ELEVATION_START => "EL",
+                FILENAME => "GSDFILE",
+                FILTER => "FILTER",
+                FREQUENCY_RESOLUTION => "FREQRES",
+                FRONTEND => "FRONTEND",
+                HUMIDITY => "HUMIDITY",
+                NUMBER_OF_CYCLES => "NOCYCLES",
+                NUMBER_OF_SUBSCANS => "NOSCANS",
+                OBJECT => "OBJECT",
+                OBSERVATION_MODE => "OBSMODE",
+                OBSERVATION_NUMBER => "SCAN",
+                PROJECT => "PROJID",
+                #        RA_BASE => "C4RADATE",
+                RECEIVER_TEMPERATURE => "TRX",
+                ROTATION => "YPOSANG",
+                REST_FREQUENCY => "RESTFRQ1",
+                SEEING => "PHA",
+                SWITCH_MODE => "SWMODE",
+                SYSTEM_TEMPERATURE => "STSYS",
+                TAU => "TAU",
+                USER_AZ_CORRECTION => "UXPNT",
+                USER_EL_CORRECTION => "UYPNT",
+                VELOCITY => "VELOCITY",
+                VELOCITY_REFERENCE_FRAME => "VREF",
+                VELOCITY_TYPE => "VDEF",
+                X_BASE => "XREF",
+                Y_BASE => "YREF",
+                X_DIM => "NOXPTS",
+                Y_DIM => "NOYPTS",
+                X_REQUESTED => "XSOURCE",
+                Y_REQUESTED => "YSOURCE",
+                X_SCALE => "DELTAX",
+                Y_SCALE => "DELTAY",
+               );
 
 # Create the translation methods
 __PACKAGE__->_generate_lookup_methods( \%CONST_MAP, \%UNIT_MAP, \@NULL_MAP );
@@ -103,8 +101,8 @@ Returns true if the supplied headers can be handled by this class.
 
   $cando = $class->can_translate( \%hdrs );
 
-For this class, the method will return true if the B<FRONTEND> header exists
-and matches the regular expression C</^rx(a|b|w)/i>.
+For this class, the method will return true if the "GSDFILE" header
+exists and the "SCA#" header exist.
 
 =cut
 
@@ -112,14 +110,10 @@ sub can_translate {
   my $self = shift;
   my $headers = shift;
 
-  if( exists( $headers->{'FRONTEND'} ) &&
-      defined( $headers->{'FRONTEND'} ) &&
-      ( $headers->{'FRONTEND'} =~ /^rx(a|b|w)/i ||
-        $headers->{'FRONTEND'} =~ /^fts/i ) ) {
+  if (exists $headers->{GSDFILE} && exists $headers->{"SCA#"}) {
     return 1;
-  } else {
-    return 0;
   }
+  return 0;
 }
 
 =back
@@ -147,77 +141,70 @@ sub to_INSTRUMENT {
   my $FITS_headers = shift;
   my $return;
 
-  if( exists( $FITS_headers->{'FRONTEND'} ) ) {
+  if ( exists( $FITS_headers->{'FRONTEND'} ) ) {
     $return = $FITS_headers->{'FRONTEND'};
-    if( $return =~ /^rxa3/i ) {
+    if ( $return =~ /^rxa3/i ) {
       $return = "RXA3";
-    } elsif( $return =~ /^rxb/i ) {
+    } elsif ( $return =~ /^rxb/i ) {
       $return = "RXB3";
     }
   }
   return $return;
 }
 
+=item B<to_OBSERVATION_ID>
+
+Calculate a unique Observation ID.
+
+=cut
+
+# Note this routine is generic for JCMT heterodyne instrumentation.
+# Would be completely generic if BACKEND was not used in preference to instrument.
+
+sub to_OBSERVATION_ID {
+  my $self = shift;
+  my $FITS_headers = shift;
+  my $backend = lc( $self->to_BACKEND( $FITS_headers ) );
+  my $obsnum = $self->to_OBSERVATION_NUMBER( $FITS_headers );
+  my $dateobs = $self->to_UTSTART( $FITS_headers );
+  my $datetime = $dateobs->datetime;
+  $datetime =~ s/-//g;
+  $datetime =~ s/://g;
+
+  my $obsid = join('_', $backend, $obsnum, $datetime);
+  return $obsid;
+}
+
 =item B<to_UTDATE>
 
-Translates the C<LONGDATE> header into a C<Time::Piece> object.
+Translates the C<DATE_OBS> or C<LONGDATEOBS> header into a
+YYYYMMDD integer.
 
 =cut
 
 sub to_UTDATE {
   my $self = shift;
   my $FITS_headers = shift;
-
-  # The UT header is in Sybase format, which is something like
-  # "Mar 15 2002  7:04:35:234AM   ". We first need to remove the number
-  # of milliseconds, then the whitespace at the end, then use the
-  # "%b%t%d%t%Y%t%T%p" format.
-
-  my $return;
-
-  if( exists( $FITS_headers->{'LONGDATE'} ) ) {
-    my $longdate = $FITS_headers->{'LONGDATE'};
-    $longdate =~ s/:\d\d\d//;
-    $longdate =~ s/\s*$//;
-    $longdate =~ s/\s*\d\d?:\d\d:\d\d[A|P]M$//;
-    $return = Time::Piece->strptime( $longdate,
-                                     "%b%t%d%t%Y" );
-  }
-
- return $return;
+  my $date = _sybase_convert_date( _date_header( $FITS_headers ), 1);
+  return $date->strftime('%Y%m%d');
 }
 
 =item B<to_UTSTART>
 
-Translates the C<LONGDATE> header into a C<Time::Piece> object.
+Translates the DB date header into a C<Time::Piece> object.
 
 =cut
 
 sub to_UTSTART {
   my $self = shift;
   my $FITS_headers = shift;
-
-  # The UT header is in Sybase format, which is something like
-  # "Mar 15 2002  7:04:35:234AM   ". We first need to remove the number
-  # of milliseconds, then the whitespace at the end, then use the
-  # "%b%t%d%t%Y%t%I:%M:%S%p" format.
-
-  my $return;
-
-  if( exists( $FITS_headers->{'LONGDATE'} ) ) {
-    my $longdate = $FITS_headers->{'LONGDATE'};
-    $longdate =~ s/:\d\d\d//;
-    $longdate =~ s/\s*$//;
-    $return = Time::Piece->strptime( $longdate,
-                                     "%b%t%d%t%Y%t%I:%M:%S%p" );
-  }
-
-  return $return;
+  return _sybase_convert_date( _date_header( $FITS_headers ));
 }
 
 =item B<to_UTEND>
 
-Translates the C<LONGDATE> header into a C<Time::Piece> object.
+Translates the database date header into a C<Time::Piece> object and adds 
+on the exposure time.
 
 =cut
 
@@ -225,27 +212,12 @@ sub to_UTEND {
   my $self = shift;
   my $FITS_headers = shift;
 
-  # The UT header is in Sybase format, which is something like
-  # "Mar 15 2002  7:04:35:234AM   ". We first need to remove the number
-  # of milliseconds, then the whitespace at the end, then use the
-  # "%b%t%d%t%Y%t%I:%M:%S%p" format.
-
-  my ($return, $expt);
-
-  if( exists( $FITS_headers->{'LONGDATE'} ) ) {
-    my $longdate = $FITS_headers->{'LONGDATE'};
-    $longdate =~ s/:\d\d\d//;
-    $longdate =~ s/\s*$//;
-    $return = Time::Piece->strptime( $longdate,
-                                     "%b%t%d%t%Y%t%I:%M:%S%p" );
-  }
-
-  $expt = $self->to_EXPOSURE_TIME( $FITS_headers );
+  my $return = _sybase_convert_date( _date_header( $FITS_headers ) );
+  return undef unless defined $return;
+  my $expt = $self->to_EXPOSURE_TIME( $FITS_headers );
 
   $return += $expt;
-
   return $return;
-
 }
 
 =item B<to_BANDWIDTH_MODE>
@@ -267,13 +239,13 @@ sub to_BANDWIDTH_MODE {
 
   my $return;
 
-  if( exists( $FITS_headers->{'NORSECT'} ) && defined( $FITS_headers->{'NORSECT'} ) &&
-      exists( $FITS_headers->{'NOFCHAN'} ) && defined( $FITS_headers->{'NOFCHAN'} ) &&
-      exists( $FITS_headers->{'NOBCHAN'} ) && defined( $FITS_headers->{'NOBCHAN'} ) ) {
+  if ( exists( $FITS_headers->{'NORSECT'} ) && defined( $FITS_headers->{'NORSECT'} ) &&
+       exists( $FITS_headers->{'NOFCHAN'} ) && defined( $FITS_headers->{'NOFCHAN'} ) &&
+       exists( $FITS_headers->{'NOBCHAN'} ) && defined( $FITS_headers->{'NOBCHAN'} ) ) {
 
     my $bandwidth = 125 * $FITS_headers->{'NORSECT'} / $FITS_headers->{'NOFCHAN'};
 
-    if( $bandwidth >= 1000 ) {
+    if ( $bandwidth >= 1000 ) {
       $bandwidth /= 1000;
       $return = sprintf( "%dGHzx%d", $bandwidth, $FITS_headers->{'NOBCHAN'} );
     } else {
@@ -295,15 +267,15 @@ sub to_EXPOSURE_TIME {
   my $FITS_headers = shift;
   my $expt = 0;
 
-  if( exists( $FITS_headers->{'OBSMODE'} ) && defined( $FITS_headers->{'OBSMODE'} ) ) {
+  if ( exists( $FITS_headers->{'OBSMODE'} ) && defined( $FITS_headers->{'OBSMODE'} ) ) {
 
     my $obsmode = uc( $FITS_headers->{'OBSMODE'} );
 
-    if( $obsmode eq 'RASTER' ) {
+    if ( $obsmode eq 'RASTER' ) {
 
-      if( exists( $FITS_headers->{'NSCAN'} ) && defined( $FITS_headers->{'NSCAN'} ) &&
-          exists( $FITS_headers->{'CYCLLEN'} ) && defined( $FITS_headers->{'CYCLLEN'} ) &&
-          exists( $FITS_headers->{'NOCYCPTS'} ) && defined( $FITS_headers->{'NOCYCPTS'} ) ) {
+      if ( exists( $FITS_headers->{'NSCAN'} ) && defined( $FITS_headers->{'NSCAN'} ) &&
+           exists( $FITS_headers->{'CYCLLEN'} ) && defined( $FITS_headers->{'CYCLLEN'} ) &&
+           exists( $FITS_headers->{'NOCYCPTS'} ) && defined( $FITS_headers->{'NOCYCPTS'} ) ) {
 
         my $nscan = $FITS_headers->{'NSCAN'};
         my $cycllen = $FITS_headers->{'CYCLLEN'};
@@ -312,43 +284,43 @@ sub to_EXPOSURE_TIME {
         # raster.
         $expt = 15 + $nscan * $cycllen * ( 1 + 1 / sqrt( $nocycpts ) ) * 1.4;
       }
-    } elsif( $obsmode eq 'PATTERN' or $obsmode eq 'GRID' ) {
+    } elsif ( $obsmode eq 'PATTERN' or $obsmode eq 'GRID' ) {
 
       my $swmode = '';
 
-      if( exists( $FITS_headers->{'SWMODE'} ) && defined( $FITS_headers->{'SWMODE'} ) ) {
+      if ( exists( $FITS_headers->{'SWMODE'} ) && defined( $FITS_headers->{'SWMODE'} ) ) {
         $swmode = $FITS_headers->{'SWMODE'};
       } else {
         $swmode = 'BEAMSWITCH';
       }
 
-      if( exists( $FITS_headers->{'NSCAN'} ) && defined( $FITS_headers->{'NSCAN'} ) &&
-          exists( $FITS_headers->{'NCYCLE'} ) && defined( $FITS_headers->{'NCYCLE'} ) &&
-          exists( $FITS_headers->{'CYCLLEN'} ) && defined( $FITS_headers->{'CYCLLEN'} ) ) {
+      if ( exists( $FITS_headers->{'NSCAN'} ) && defined( $FITS_headers->{'NSCAN'} ) &&
+           exists( $FITS_headers->{'NCYCLE'} ) && defined( $FITS_headers->{'NCYCLE'} ) &&
+           exists( $FITS_headers->{'CYCLLEN'} ) && defined( $FITS_headers->{'CYCLLEN'} ) ) {
 
         my $nscan = $FITS_headers->{'NSCAN'};
         my $ncycle = $FITS_headers->{'NCYCLE'};
         my $cycllen = $FITS_headers->{'CYCLLEN'};
 
-        if( $swmode eq 'POSITION_SWITCH' ) {
+        if ( $swmode eq 'POSITION_SWITCH' ) {
 
           # position switch pattern/grid.
           $expt = 6 + $nscan * $ncycle * $cycllen * 1.35;
 
-        } elsif( $swmode eq 'BEAMSWITCH' ) {
+        } elsif ( $swmode eq 'BEAMSWITCH' ) {
 
           # beam switch pattern/grid.
           $expt = 6 + $nscan * $ncycle * $cycllen * 1.35;
 
-        } elsif( $swmode eq 'CHOPPING' ) {
-          if( exists( $FITS_headers->{'FRONTEND'} ) && defined( $FITS_headers->{'FRONTEND'} ) ) {
+        } elsif ( $swmode eq 'CHOPPING' ) {
+          if ( exists( $FITS_headers->{'FRONTEND'} ) && defined( $FITS_headers->{'FRONTEND'} ) ) {
             my $frontend = uc( $FITS_headers->{'FRONTEND'} );
-            if( $frontend eq 'RXA3I' ) {
+            if ( $frontend eq 'RXA3I' ) {
 
               # fast frequency switch pattern/grid, receiver A.
               $expt = 15 + $nscan * $ncycle * $cycllen * 1.20;
 
-            } elsif( $frontend eq 'RXB' ) {
+            } elsif ( $frontend eq 'RXB' ) {
 
               # slow frequency switch pattern/grid, receiver B.
               $expt = 18 + $nscan * $ncycle * $cycllen * 1.60;
@@ -360,39 +332,39 @@ sub to_EXPOSURE_TIME {
     } else {
 
       my $swmode;
-      if( exists( $FITS_headers->{'SWMODE'} ) && defined( $FITS_headers->{'SWMODE'} ) ) {
+      if ( exists( $FITS_headers->{'SWMODE'} ) && defined( $FITS_headers->{'SWMODE'} ) ) {
         $swmode = uc( $FITS_headers->{'SWMODE'} );
       } else {
         $swmode = 'BEAMSWITCH';
       }
 
-      if( exists( $FITS_headers->{'NSCAN'} ) && defined( $FITS_headers->{'NSCAN'} ) &&
-          exists( $FITS_headers->{'NCYCLE'} ) && defined( $FITS_headers->{'NCYCLE'} ) &&
-          exists( $FITS_headers->{'CYCLLEN'} ) && defined( $FITS_headers->{'CYCLLEN'} ) ) {
+      if ( exists( $FITS_headers->{'NSCAN'} ) && defined( $FITS_headers->{'NSCAN'} ) &&
+           exists( $FITS_headers->{'NCYCLE'} ) && defined( $FITS_headers->{'NCYCLE'} ) &&
+           exists( $FITS_headers->{'CYCLLEN'} ) && defined( $FITS_headers->{'CYCLLEN'} ) ) {
 
         my $nscan = $FITS_headers->{'NSCAN'};
         my $ncycle = $FITS_headers->{'NCYCLE'};
         my $cycllen = $FITS_headers->{'CYCLLEN'};
 
-        if( $swmode eq 'POSITION_SWITCH' ) {
+        if ( $swmode eq 'POSITION_SWITCH' ) {
 
           # position switch sample.
           $expt = 4.8 + $nscan * $ncycle * $cycllen * 1.10;
 
-        } elsif( $swmode eq 'BEAMSWITCH' ) {
+        } elsif ( $swmode eq 'BEAMSWITCH' ) {
 
           # beam switch sample.
           $expt = 4.8 + $nscan * $ncycle * $cycllen * 1.25;
 
-        } elsif( $swmode eq 'CHOPPING' ) {
-          if( exists( $FITS_headers->{'FRONTEND'} ) && defined( $FITS_headers->{'FRONTEND'} ) ) {
+        } elsif ( $swmode eq 'CHOPPING' ) {
+          if ( exists( $FITS_headers->{'FRONTEND'} ) && defined( $FITS_headers->{'FRONTEND'} ) ) {
             my $frontend = uc( $FITS_headers->{'FRONTEND'} );
-            if( $frontend eq 'RXA3I' ) {
+            if ( $frontend eq 'RXA3I' ) {
 
               # fast frequency switch sample, receiver A.
               $expt = 3 + $nscan * $ncycle * $cycllen * 1.10;
 
-            } elsif( $frontend eq 'RXB' ) {
+            } elsif ( $frontend eq 'RXB' ) {
 
               # slow frequency switch sample, receiver B.
               $expt = 3 + $nscan * $ncycle * $cycllen * 1.40;
@@ -416,8 +388,8 @@ sub to_SYSTEM_VELOCITY {
   my $self = shift;
   my $FITS_headers = shift;
   my $return;
-  if( exists( $FITS_headers->{'VREF'} ) && defined( $FITS_headers->{'VREF'} ) &&
-      exists( $FITS_headers->{'VDEF'} ) && defined( $FITS_headers->{'VDEF'} ) ) {
+  if ( exists( $FITS_headers->{'VREF'} ) && defined( $FITS_headers->{'VREF'} ) &&
+       exists( $FITS_headers->{'VDEF'} ) && defined( $FITS_headers->{'VDEF'} ) ) {
     $return = uc( substr( $FITS_headers->{'VDEF'}, 0, 3 ) . substr( $FITS_headers->{'VREF'}, 0, 3 ) );
   }
   return $return;
@@ -425,9 +397,77 @@ sub to_SYSTEM_VELOCITY {
 
 =back
 
+=begin __PRIVATE
+
+=over 4
+
+=item B<_date_header>
+
+Works out which header corresponds to the date field in sybase format and returns the value.
+
+  $value = _date_header( $FITS_headers );
+
+Returns undef if none found.
+
+=cut
+
+sub _date_header {
+  my $FITS_headers = shift;
+  for my $key (qw/ LONGDATEOBS DATE_OBS LONGDATE/ ) {
+    if (exists $FITS_headers->{$key} && defined $FITS_headers->{$key}) {
+      return $FITS_headers->{$key};
+    }
+  }
+  return;
+}
+
+=item B<_sybase_convert_date>
+
+Converts a sybase long date to Time::Piece object.
+
+  $date = _sybase_convert_date( $string );
+
+Optional flag can be set to true to drop Hours, minutes and seconds from parse.
+
+  $utday = _sybase_convert_date( $string, 1);
+
+Returns undef if no string is supplied.
+
+=cut
+
+sub _sybase_convert_date {
+  my $longdate = shift;
+  my $drophms = shift;
+  return undef unless $longdate;
+
+  # The UT header is in Sybase format, which is something like
+  # "Mar 15 2002  7:04:35:234AM   ". We first need to remove the number
+  # of milliseconds, then the whitespace at the end, then use the
+  # "%b%t%d%t%Y%t%I:%M:%S%p" format.
+
+  # General cleanup
+  $longdate =~ s/:\d\d\d//;
+  $longdate =~ s/\s*$//;
+
+  my $return;
+  if ($drophms) {
+    $longdate =~ s/\s*\d\d?:\d\d:\d\d[A|P]M$//;
+    $return = Time::Piece->strptime( $longdate,
+                                     "%b %e %Y" );
+  } else {
+    $return = Time::Piece->strptime( $longdate,
+                                     "%b %e %Y %l:%M:%S%p" );
+  }
+  return $return;
+}
+
+=back
+
+=end __PRIVATE
+
 =head1 REVISION
 
-$Id: JCMT_GSD_DB.pm,v 1.12 2006/02/08 21:03:46 bradc Exp $
+$Id$
 
 =head1 AUTHOR
 
@@ -436,7 +476,8 @@ Tim Jenness E<lt>t.jenness@jach.hawaii.eduE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2003-2005 Particle Physics and Astronomy Research Council.
+Copyright (C) 2008 Science and Technology Facilities Council.
+Copyright (C) 2003-2007 Particle Physics and Astronomy Research Council.
 All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
