@@ -1,5 +1,3 @@
-# -*-perl-*-
-
 package Astro::FITS::HdrTrans::UKIRTNew;
 
 =head1 NAME
@@ -27,80 +25,61 @@ use Carp;
 # Inherit from UKIRT
 use base qw/ Astro::FITS::HdrTrans::UKIRT /;
 
-# Use the FITS standard DATE-OBS handling
-use Astro::FITS::HdrTrans::FITS;
-
 use vars qw/ $VERSION /;
 
-$VERSION = sprintf("%d.%03d", q$Revision: 1.1 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "1.02";
 
-# for a constant mapping, there is no FITS header, just a generic
-# header that is constant
+# For a constant mapping, there is no FITS header, just a generic
+# header that is constant.
 my %CONST_MAP = (
 
-		);
-
-# unit mapping implies that the value propogates directly
-# to the output with only a keyword name change
+                );
 
+# Unit mapping implies that the value propogates directly
+# to the output with only a keyword name change.
 my %UNIT_MAP = (
-		DEC_TELESCOPE_OFFSET => "TDECOFF",
-		DR_RECIPE            => "RECIPE",
-		GAIN                 => "GAIN",
-		RA_TELESCOPE_OFFSET  => "TRAOFF",
-	       );
+                DEC_TELESCOPE_OFFSET => "TDECOFF",
+                DR_RECIPE            => "RECIPE",
+                EXPOSURE_TIME        => "EXP_TIME",
+                GAIN                 => "GAIN",
+                RA_TELESCOPE_OFFSET  => "TRAOFF",
+                UTDATE               => "UTDATE",
+               );
 
-# Create the translation methods
+# Create the translation methods.
 __PACKAGE__->_generate_lookup_methods( \%CONST_MAP, \%UNIT_MAP );
 
 =head1 COMPLEX CONVERSIONS
 
-These methods are more complicated than a simple mapping. We have to
-provide both from- and to-FITS conversions All these routines are
+These methods are more complicated than a simple mapping.  We have to
+provide both from- and to-FITS conversions.  All these routines are
 methods and the to_ routines all take a reference to a hash and return
-the translated value (a many-to-one mapping) The from_ methods take a
-reference to a generic hash and return a translated hash (sometimes
-these are many-to-many)
+the translated value (a many-to-one mapping).  The from_ methods take
+a reference to a generic hash and return a translated hash (sometimes
+these are many-to-many).
 
 =over 4
 
-=item B<to_UTDATE>
+=item B<to_INST_DHS>
 
-Converts FITS header values into one unified UT start date value.
+Sets the instrument data-handling-system header.
 
 =cut
 
-sub to_UTDATE {
+sub to_INST_DHS {
   my $self = shift;
   my $FITS_headers = shift;
   my $return;
-  if(exists($FITS_headers->{UTDATE})) {
-    my $utdate = $FITS_headers->{UTDATE};
-    $return = Time::Piece->strptime( $utdate, "%Y%m%d" );
+
+  if ( exists( $FITS_headers->{DHSVER} ) ) {
+    $FITS_headers->{DHSVER} =~ /^(\w+)/;
+    my $dhs = uc( $1 );
+    $return = $FITS_headers->{INSTRUME} . "_$dhs";
   }
 
   return $return;
 }
 
-=item B<from_UTDATE>
-
-Converts UT date in the form C<yyyy-mm-dd> to C<yyyymmdd>.
-
-=cut
-
-sub from_UTDATE {
-  my $self = shift;
-  my $generic_headers = shift;
-  my %return_hash;
-  if( exists( $generic_headers->{UTDATE} ) ) {
-    my $date = $generic_headers->{UTDATE};
-    if( ! UNIVERSAL::isa( $date, "Time::Piece" ) ) { return; }
-    $return_hash{UTDATE} = sprintf("%4d%02d%02d", $date->year, $date->mon, $date->mday);
-
-  }
-  return %return_hash;
-}
-
 =item B<to_UTSTART>
 
 Converts UT date in C<DATE-OBS> header into C<Time::Piece> object.
@@ -110,16 +89,13 @@ Converts UT date in C<DATE-OBS> header into C<Time::Piece> object.
 sub to_UTSTART {
   my $self = shift;
   my $FITS_headers = shift;
-  my $return;
-  if(exists($FITS_headers->{'DATE-OBS'})) {
-    my $utstart = $FITS_headers->{'DATE-OBS'};
-    $utstart =~ s/Z//g;
-
-    # use the standard FITS parser now that we have dropped the Z
-    $return = Astro::FITS::HdrTrans::FITS->to_UTSTART( { 'DATE-OBS' => 
-							   $utstart});
-  }
-  return $return;
+  my $dateobs = (exists $FITS_headers->{"DATE-OBS"} ?
+                 $FITS_headers->{"DATE-OBS"} : undef );
+  my @rutstart = sort {$a<=>$b} $self->via_subheader( $FITS_headers, "UTSTART" );
+  my $utstart = $rutstart[0];
+  return $self->_parse_date_info( $dateobs,
+                                  $self->to_UTDATE( $FITS_headers ),
+                                  $utstart );
 }
 
 =item B<from_UTSTART>
@@ -129,18 +105,19 @@ YYYY-MM-DDThh:mm:ss.
 
 =cut
 
-
 sub from_UTSTART {
   my $self = shift;
   my $generic_headers = shift;
 
-  # use the FITS standard parser
-  my %return_hash = Astro::FITS::HdrTrans::FITS->from_UTSTART( $generic_headers);
+  # Use the FITS standard parser.
+  my %return_hash = Astro::FITS::HdrTrans::FITS->from_UTSTART( $generic_headers );
 
-  if (exists $return_hash{'DATE-OBS'}) {
-    # prior to April 2005 the UKIRT FITS headers had a trailing Z
-    # Part of the ISO8601 standard but not part of the FITS standard
-    # (which always assumes UTC)
+  if ( exists $return_hash{'DATE-OBS'} ) {
+
+    # Prior to April 2005 the UKIRT FITS headers had a trailing Z.
+    # This is part of the ISO8601 standard but not part of the FITS
+    # standard (which always assumes UTC), although it was in the
+    # draft FITS agreement.
     $return_hash{'DATE-OBS'} .= "Z"
       if $generic_headers->{UTSTART}->epoch < 1112662116;
   }
@@ -156,16 +133,14 @@ Converts UT date in C<DATE-END> header into C<Time::Piece> object.
 sub to_UTEND {
   my $self = shift;
   my $FITS_headers = shift;
-  my $return;
-  if(exists($FITS_headers->{'DATE-END'})) {
-    my $utstart = $FITS_headers->{'DATE-END'};
-    $utstart =~ s/Z//g;
+  my $dateend = (exists $FITS_headers->{"DATE-END"} ?
+                 $FITS_headers->{"DATE-END"} : undef );
 
-    # use the standard FITS parser now that we have dropped the Z
-    $return = Astro::FITS::HdrTrans::FITS->to_UTEND( { 'DATE-END' => 
-						       $utstart});
-  }
-  return $return;
+  my @rutend = sort {$a<=>$b} $self->via_subheader( $FITS_headers, "UTEND" );
+  my $utend = $rutend[-1];
+  return $self->_parse_date_info( $dateend,
+                                  $self->to_UTDATE( $FITS_headers ),
+                                  $utend );
 }
 
 =item B<from_UTEND>
@@ -175,50 +150,30 @@ YYYY-MM-DDThh:mm:ss.
 
 =cut
 
-
 sub from_UTEND {
   my $self = shift;
   my $generic_headers = shift;
 
-  # use the FITS standard parser
+  # Use the FITS standard parser.
   my %return_hash = Astro::FITS::HdrTrans::FITS->from_UTEND( $generic_headers);
 
-  if (exists $return_hash{'DATE-END'}) {
-    # prior to April 2005 the UKIRT FITS headers had a trailing Z
-    # Part of the ISO8601 standard but not part of the FITS standard
-    # (which always assumes UTC)
+  if ( exists $return_hash{'DATE-END'} ) {
+
+    # Prior to April 2005 the UKIRT FITS headers had a trailing Z.
+    # This is part of the ISO8601 standard but not part of the FITS
+    # standard (which always assumes UTC), although it was in the
+    # draft FITS agreement.
     $return_hash{'DATE-END'} .= "Z"
       if $generic_headers->{UTEND}->epoch < 1112662116;
   }
   return %return_hash;
 }
 
-=item B<to_INST_DHS>
-
-Sets the instrument data handling system header.
-
-=cut
-
-sub to_INST_DHS {
-  my $self = shift;
-  my $FITS_headers = shift;
-  my $return;
-
-  if( exists( $FITS_headers->{DHSVER} ) ) {
-    $FITS_headers->{DHSVER} =~ /^(\w+)/;
-    my $dhs = uc($1);
-    $return = $FITS_headers->{INSTRUME} . "_$dhs";
-  }
-
-  return $return;
-
-}
-
 =back
 
 =head1 REVISION
 
- $Id: UKIRTNew.pm,v 1.1 2005/04/06 03:42:10 timj Exp $
+ $Id$
 
 =head1 SEE ALSO
 
@@ -228,15 +183,17 @@ C<Astro::FITS::HdrTrans>, C<Astro::FITS::HdrTrans::UKIRT>
 
 Brad Cavanagh E<lt>b.cavanagh@jach.hawaii.eduE<gt>,
 Tim Jenness E<lt>t.jenness@jach.hawaii.eduE<gt>.
+Malcolm J. Currie E<lt>mjc@star.rl.ac.ukE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2003-2005 Particle Physics and Astronomy Research Council.
+Copyright (C) 2007-2008 Science and Technology Facilities Council.
+Copyright (C) 2003-2007 Particle Physics and Astronomy Research Council.
 All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
+Foundation; either Version 2 of the License, or (at your option) any later
 version.
 
 This program is distributed in the hope that it will be useful,but WITHOUT ANY
@@ -245,7 +202,7 @@ PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place,Suite 330, Boston, MA  02111-1307, USA
+Place, Suite 330, Boston, MA  02111-1307, USA.
 
 =cut
 
